#ifndef __EOPT_HH__
#define __ETOP_HH__

#include <vector>
#include "opt.hh"
#include "grid.hh"
#include "pudag.hh"
#include "epnode.hh"
#include "zset.hh"
#include "matcher.hh"
#include "flags.hh"

//Energy optimization

class EOptimist : public Optimizer {

public:
  EOptimist(Architecture *, float _delta_e = 1.0);
  ~EOptimist();
  Schedule *schedule(DAG &, Flags &);

private:
  Architecture *architecture;

  //Solution space
  static const int GRID_DIM = 2;
  static const int E_AXIS = 0;
  static const int L_AXIS = 1;

  typedef grid< ENode *, GRID_DIM > solution_space_t;
  typedef std::vector< unsigned int > coord_t;
#ifdef __GNUC__
  typedef __gnu_cxx::hash_set< ENode *, __gnu_cxx::hash< ENode * >, std::equal_to< ENode * > > grid_elem_t;
#else
  typedef std::hash_set< ENode *, std::hash< ENode * >, std::equal_to< ENode * > > grid_elem_t;
#endif //__GNUC__

  NodeAttr *get_init_node_attr(ZSet &, DAG &);
  NodeAttr *select(const vertex_set_t & _chi, const NodeAttr &, ZSet &, DAG &);
  int reduce_init_enode(ENode &_enode, Matcher _matcher, ZSet & _zset, DAG & _dag);
  void update_space(const ENode *, TargetInstruction *,
		    const vertex_set_t &, const coord_t &, 
		    solution_space_t & _solution_space,
		    coord_t & _max_coord,
		    vertex_id & _root, node_map_t & _arg_map,
		    pattern_t & _pattern, ZSet & _zset,
		    DAG & _dag);
  float delta_e;
  int cut, improved, merged, n_enodes;

};

//Equality and hashing for the grid
#ifdef __GNUC__
namespace __gnu_cxx 
#else
namespace std
#endif //__GNUC__
{
  template<>
  struct hash< ENode * >
  {
    size_t operator()(const ENode * n) const
    {
      return n->get_hash();
    }
  };
}; //std or __gnu_cxx

namespace std {
  template<> 
  struct equal_to< ENode * >
  {
    bool operator()(const ENode * n1, const ENode * n2) const
    {
      return *n1 == *n2;
    }
  };
}; //std

#endif //__EOPT_HH__
